#ifndef EngErrors_H
#define EngErrors_H

// Error Engine

#include <QObject>
#include <QString>
#include <QDateTime>
#include <QMessageBox>
#include <exception>
#include <QtDebug>

extern int gErrorLimit;

class Error
{
public:
   enum eErrorType
   {
      eInformation,
      eWarning,
      eCritical
   };

   enum eErrorCode
   {
      eUnknownError = 100000
   };


   eErrorType mType;
   int mCode;
   QString mMessage;
   QDateTime mDateTime;


   Error(eErrorType aType, QString aMessage, int aCode = eUnknownError)
   {
      mType = aType;
      mCode = aCode;
      mMessage = aMessage;
      mDateTime = QDateTime::currentDateTime();
   }

};


class FacErrors : public QObject
{
   Q_OBJECT

public:
   static FacErrors* instance();
   FacErrors(){}
   ~FacErrors();

   void error(Error::eErrorType aType, QString aMessage, int aCode = Error::eUnknownError, bool aException = false);
   QList<Error*> errors();
public slots:
   void clearErrorList();
signals:
   void newError(Error* aError);

protected:
   QList<Error*> mErrors;
};


class Exception : public std::exception
{
public:
   Exception(QString aMessage, int aCode = Error::eUnknownError) : std::exception(), mMessage(aMessage), mCode(aCode) {}
   virtual ~Exception() throw() {}
   QString mMessage;
   int mCode;
   virtual const char* what() const throw() { return QString("Error code: %1\nMessage:\n%2").arg(mCode).arg(mMessage).toStdString().c_str(); }
};

inline void gInformation(QString aMessage, int aCode = Error::eUnknownError)
{
    qInfo() << QString("%1").arg(aCode) << ":" << aMessage;
   FacErrors::instance()->error(Error::eInformation, aMessage, aCode, false);
}

inline void gInformationException(QString aMessage, int aCode = Error::eUnknownError)
{
    qInfo() << QString("%1").arg(aCode) << ":" << aMessage;
   FacErrors::instance()->error(Error::eInformation, aMessage, aCode, true);
}

inline void gWarning(QString aMessage, int aCode = Error::eUnknownError)
{
    qWarning() << QString("%1").arg(aCode) << ":" << aMessage;
   FacErrors::instance()->error(Error::eWarning, aMessage, aCode, false);
}

inline void gWarningException(QString aMessage, int aCode = Error::eUnknownError)
{
    qWarning() << QString("%1").arg(aCode) << ":" << aMessage;
   FacErrors::instance()->error(Error::eWarning, aMessage, aCode, true);
}

inline void gCritical(QString aMessage, int aCode = Error::eUnknownError)
{
    qCritical() << QString("%1").arg(aCode) << ":" << aMessage;
   FacErrors::instance()->error(Error::eCritical, aMessage, aCode, false);
}

inline void gCriticalException(QString aMessage, int aCode = Error::eUnknownError)
{
    qCritical() << QString("%1").arg(aCode) << ":" << aMessage;
   FacErrors::instance()->error(Error::eCritical, aMessage, aCode, true);
}


inline void gMessage(QString aShortText, QString aInformativeText = "", QString aDetailedText = "", Error::eErrorType aType = Error::eInformation)
{
   QMessageBox lMessage;

   lMessage.setText(aShortText);
   lMessage.setInformativeText(aInformativeText);
   lMessage.setDetailedText(aDetailedText);
   QMessageBox::Icon lIcon = QMessageBox::NoIcon;
   if(aType == Error::eInformation)
      lIcon = QMessageBox::Information;
   else if(aType == Error::eWarning)
      lIcon = QMessageBox::Warning;
   else if(aType == Error::eCritical)
      lIcon = QMessageBox::Critical;
   lMessage.setIcon(lIcon);

   lMessage.exec();
}

inline void gMessage(QString aShortText, Error::eErrorType aType)
{
    gMessage(aShortText, aShortText, aShortText, aType);
}





#endif // EngErrors_H
