#include <QApplication>
#include <iostream>
#include <QDateTime>
#include <omniORB4/CORBA.h>
#include "WinMain.h"
#include "UserImp.h"
#include "FacErrors.h"

using namespace std;

int main(int argc, char *argv[])
{
    try
    {
        qsrand(QDateTime::currentDateTime().toTime_t());

        CORBA::ORB_var lORB;
        PortableServer::POA_var lPoa;

        lORB = CORBA::ORB_init(argc, argv);
        CORBA::Object_var lPOAObj = lORB->resolve_initial_references("RootPOA");
        lPoa = PortableServer::POA::_narrow(lPOAObj);
        PortableServer::POAManager_var lPOAManager = lPoa->the_POAManager();
        lPOAManager->activate();

        CosNaming::NamingContext_var lNamingContext;
        CosNaming::Name lName;

        CORBA::Object_var lNameServiceObj = lORB->resolve_initial_references("NameService");
        if(CORBA::is_nil(lNameServiceObj))
            throw Exception("lNameServiceObj is nil.");
        lNamingContext = CosNaming::NamingContext::_narrow(lNameServiceObj);

        lName.length(1);
        lName[0].id = CORBA::string_dup("FacRoom");
        lName[0].kind = CORBA::string_dup("Object");

        TCzat::FacRoom_var lFacRoom;

        lFacRoom = TCzat::FacRoom::_narrow(lNamingContext->resolve(lName));

        int lResult(0);
        {
            QApplication a(argc, argv);
            WinMainC w(0, lPoa, lFacRoom);
            w.show();
            lResult = a.exec();
        }

        lORB->destroy();

        return lResult;
    }
    catch(CORBA::TRANSIENT& aEx)
    {
        cerr << "CORBA::TRANSIENT occurs: " << aEx._name() << endl;
        return EXIT_FAILURE;
    }
    catch(CORBA::COMM_FAILURE& aEx)
    {
        cerr << "CORBA::COMM_FAILURE occurs: " << aEx._name() << endl;
        return EXIT_FAILURE;
    }
    catch(CORBA::SystemException& aEx)
    {
        cerr << "CORBA::SystemException occurs: " << aEx._name() << endl;
        return EXIT_FAILURE;
    }
    catch(PortableServer::POA::AdapterAlreadyExists& aEx)
    {
        cerr << "PortableServer::POA::AdapterAlreadyExists occurs: " << aEx._name() << endl;
        return EXIT_FAILURE;
    }
    catch(std::exception& aEx)
    {
        cerr << "Std::exception occurs: " << aEx.what() << endl;
        return EXIT_FAILURE;
    }
    catch(...)
    {
        cerr << "Unknown exception occurs!!!" << endl;
        return EXIT_FAILURE;
    }
}
