#include <iostream>
#include <memory>
#include "WinMain.h"
#include "ui_WinMain.h"
#include "WinLogin.h"
#include "FacErrors.h"

using namespace std;

WinMainC::WinMainC(QWidget *aParent, PortableServer::POA_var aPoa, TCzat::FacRoom_var aFacRoom) :
    QMainWindow(aParent)
    , ui(new Ui::WinMain)
    , mPoa(aPoa)
    , mFacRoom(aFacRoom)
    , mRoom(0)
    , mUser(0)
{
    ui->setupUi(this);
    ui->mText->setMaximumBlockCount(250);

    connect(ui->mMessage, SIGNAL(returnPressed()), this, SLOT(returnPressed()));
    connect(ui->mActionConnect, &QAction::triggered, this, &WinMainC::connectToServer);

    connectToServer();
}

WinMainC::~WinMainC()
{
    gInformation("WinMainC::~WinMainC() - begin");
    delete ui;

    disconnectFromServer();

    gInformation("WinMainC::~WinMainC() - end");
}

void WinMainC::returnPressed()
{
    if(!CORBA::is_nil(mRoomController)) mRoomController->message(mUser->mName.toStdWString().c_str(), mRoom->name(), ui->mMessage->text().toStdWString().c_str());
    //mUser->sendMessage(mUser->mName.toStdWString().c_str(), mRoom->name(), ui->mMessage->text().toStdWString().c_str());

    ui->mMessage->setText("");
}

void WinMainC::showMessage(QString aFrom, QString aTo, QString aMessage)
{
    // to do: rooms and private messages
    ui->mText->appendPlainText(aMessage);
}

void WinMainC::addUser(QString aName)
{
    ui->mUsers->insertItem(ui->mUsers->count(), aName);
    if(mUser && (mUser->mName == aName))
    {
        ui->mActionConnect->setText(tr("Disconnect"));
    }
}

void WinMainC::delUser(QString aName)
{
    QList<QListWidgetItem*> lItems = ui->mUsers->findItems(aName, Qt::MatchExactly);
    for(int i = 0; i < lItems.size(); ++i)
        ui->mUsers->removeItemWidget(lItems[i]);
    if(mUser && (mUser->mName == aName))
    {
        delete mUser;
        mUser = 0;
    }
}

void WinMainC::connectToServer()
{
    auto_ptr<WinLogin> lLogin(new WinLogin(0));
    if(!lLogin->exec()) // login aborted
        return;
    mRoom = TCzat::Room::_duplicate(mFacRoom->openRoom(lLogin->mRoomName.toStdWString().c_str()));

    if(mUser)
    {
        disconnectFromServer();
        return;
    }

    if(!mUser)
    {
        mUser = new UserImp(mPoa);
        connect(mUser, &UserImp::newMessage, this, &WinMainC::showMessage);
        connect(mUser, &UserImp::sigAddUser, this, &WinMainC::addUser);
        connect(mUser, &UserImp::sigDelUser, this, &WinMainC::delUser);
    }
    mUser->mRoom = lLogin->mRoomName;
    mUser->mName = lLogin->mUserLogin;
    mUser->mPassword = lLogin->mPassword;
    mUser->mUserObjectID = mPoa->activate_object(mUser);
    TCzat::User_var lUserPtr = mUser->_this();
    mRoomController = mRoom->joinUser(lUserPtr);
    //TCzat::RoomController_var lRoomController = mRoom->joinUser(lUserPtr);
    //mUser->mRoomController = TCzat::RoomController::_duplicate(lRoomController);
}

void WinMainC::disconnectFromServer()
{
    if(!CORBA::is_nil(mRoomController))
    {
        if(mUser)
        {
            TCzat::User_var lUser = mUser->_this();
            mRoomController->removeUser(lUser);
        }
        mRoomController->destroy();
//        CORBA::release(mRoomController);
        mRoomController = TCzat::RoomController::_nil();
    }

    if(mUser)
    {
        mUser->destroy();
        mUser = 0;
    }
}
