#include <iostream>
#include <QString>
#include "RoomImp.h"
#include "IteUserImp.h"
#include "RoomControllerImp.h"
#include "FacErrors.h"

using namespace std;

RoomImp::RoomImp(PortableServer::POA_ptr aPoa, TCzat::FacRoom_ptr aFacRoom, QString aName)
    : POA_TCzat::Room(), mPoa(PortableServer::POA::_duplicate(aPoa)), mFacRoom(TCzat::FacRoom::_duplicate(aFacRoom)), mName(aName)
{
    gInformation("RoomImp::RoomImp - begin");
    mUserIteratorPoa = mPoa->create_POA("IterPOA", aPoa->the_POAManager(), NULL);
    mUserIteratorPoa->the_POAManager()->activate();
    gInformation("RoomImp::RoomImp - end");
}

RoomImp::~RoomImp()
{
    gInformation("RoomImp::~RoomImp - begin");
    // remove users
    for(int i = 0; i < mUsers.size(); ++i)
        CORBA::release(mUsers[i]);
    mUsers.clear();

    // remove poas
    mUserIteratorPoa->the_POAManager()->deactivate(true, false);    // is this correct here???
    CORBA::release(mUserIteratorPoa);
    CORBA::release(mPoa);

    gInformation("RoomImp::~RoomImp - end");
}

::CORBA::WChar* RoomImp::name()
{
    return CORBA::wstring_dup(mName.toStdWString().c_str());
}

void RoomImp::setName(const ::CORBA::WChar* aName)
{
    mName = QString::fromWCharArray(aName);
}


TCzat::RoomController_ptr RoomImp::joinUser(::TCzat::User_ptr aUser)
{
    // add user to internal list
    mUsers.append(::TCzat::User::_duplicate(aUser));

    // call all users to add user to they list
    for(int i = 0; i < mUsers.length(); ++i)
        mUsers[i]->addUser(aUser);

    TCzat::Room_var lRoom = _this();
    RoomControllerImp* lController = new RoomControllerImp(lRoom, mPoa);
    PortableServer::ObjectId_var lControllerID = mPoa->activate_object(lController);

    TCzat::RoomController_ptr lControllerPtr = lController->_this();
    return lControllerPtr;
}

void RoomImp::removeUser(::TCzat::User_ptr aUser)
{
    // remove user from internal list
    for(int i = 0; i < mUsers.size();)
        if(QString::fromWCharArray(mUsers[i]->name()) == QString::fromWCharArray(aUser->name()))
            mUsers.removeAt(i);
        else
            ++i;

    // call all users to remove user from they list
    for(int i = 0; i < mUsers.length(); ++i)
        mUsers[i]->delUser(aUser);

    if(!mUsers.size())
    {
        TCzat::Room_ptr lRoom = _this();
        mFacRoom->closeRoom(lRoom);
    }
}

void RoomImp::message(const ::CORBA::WChar* aFrom, const ::CORBA::WChar* aTo, const ::CORBA::WChar* aMessage)
{
    if(mName == QString::fromWCharArray(aTo))   // normal public message
        for(int i = 0; i < mUsers.length(); ++i)
            mUsers[i]->showMessage(CORBA::wstring_dup(aFrom), CORBA::wstring_dup(aTo), CORBA::wstring_dup(aMessage));
    else    // private message
        for(int i = 0; i < mUsers.length(); ++i)
            if(QString::fromWCharArray(mUsers[i]->name()) == QString::fromWCharArray(aTo))
                mUsers[i]->showMessage(CORBA::wstring_dup(aFrom), CORBA::wstring_dup(aTo), CORBA::wstring_dup(aMessage));
}


inline bool operator==(const PortableServer::ObjectId_var aObjectID1, const PortableServer::ObjectId_var aObjectID2)
{
    return QString::fromLatin1((char*)aObjectID1.in().get_buffer()) == QString::fromLatin1((char*)aObjectID2.in().get_buffer());
}

inline uint qHash(const PortableServer::ObjectId_var aObjectID, uint aSeed)
{
    return qHash(QString::fromLatin1((char*)aObjectID.in().get_buffer()), aSeed);
}

TCzat::IteUser_ptr RoomImp::users()
{
    TCzat::Room_var lRoom = _this();
    IteUserImp* lIteUser = new IteUserImp(lRoom, mUserIteratorPoa, mUsers);
    PortableServer::ObjectId_var lId = mUserIteratorPoa->activate_object(lIteUser);
    TCzat::IteUser_ptr lIteUserPtr = lIteUser->_this();
    mUserIterators[lId] = TCzat::IteUser::_duplicate(lIteUserPtr);

    return lIteUserPtr;
}

void RoomImp::removeIteUser(::TCzat::IteUser_ptr aIterator)
{
    PortableServer::ObjectId_var lId = mPoa->reference_to_id(aIterator);
    mUserIterators.remove(lId);
    //CORBA::release(aIterator);
}

